/*  rs232 interface (serial port communications) without a level converter.
 *  Therefore microcontroller pins are connected DIRECTLY to the serial cable.
 *  Therefore inverted (from standard) logic is used.
 *  
 *  PIC PIN01 PORTA2 - TRANSMIT (receive at PC end of serial cable) white
 *  PIC PIN17 PORTA0 - RECEIVE (transmit at PC end of serial cable) orange
 */



#include <pic.h>
#include "delay.c"

/* Initialize the transmit pin, 
 * by ensuring it is low for an entire message cycle.
 */
 
 
unsigned char  NL = 0X0A ,CR = 0X0D;		// NEW LINE AND CARRIGE RETURN 

 
void rs232_initialize()
{
  RA2 = 0;
  delay_ms(1);
}

/* Send a single char out the transmit pin.
 */
void rs232_send_char(unsigned char c) 
{
  c = ~c;

  // start bit

  RA2 = 1;
  delay_us(100);

  // 8 data bits

  RA2 = (c & 1);
  c = c >> 1;
  delay_us(98);

  RA2 = (c & 1);
  c = c >> 1;
  delay_us(98);

  RA2 = (c & 1);
  c = c >> 1;
  delay_us(98);

  RA2 = (c & 1);
  c = c >> 1;
  delay_us(98);

  RA2 = (c & 1);
  c = c >> 1;
  delay_us(98);

  RA2 = (c & 1);
  c = c >> 1;
  delay_us(98);

  RA2 = (c & 1);
  c = c >> 1;
  delay_us(98);

  RA2 = (c & 1);
  c = c >> 1;
  delay_us(98);

  // stop bit

  RA2 = 0;
  delay_us(100);
}

/* Send a string of chars out the transmit pin.
 */
void rs232_send_string(const char * s)
{
  while(*s)
  { rs232_send_char(*s++); }
}

/* Get a single char from the receive pin.
 * Then send that char over the transmit pin.
 */
void rs232_echo_char() 
{
  unsigned char c = 0;

  // start bit

  while (RA0 == 0) { continue; }
  delay_us(125);

  // 8 data bits

  if (RA0 == 0) { c = c | 0b00000001; }
  delay_us(97);

  if (RA0 == 0) { c = c | 0b00000010; }
  delay_us(97);

  if (RA0 == 0) { c = c | 0b00000100; }
  delay_us(97);

  if (RA0 == 0) { c = c | 0b00001000; }
  delay_us(97);

  if (RA0 == 0) { c = c | 0b00010000; }
  delay_us(97);

  if (RA0 == 0) { c = c | 0b00100000; }
  delay_us(97);

  if (RA0 == 0) { c = c | 0b01000000; }
  delay_us(97);

  if (RA0 == 0) { c = c | 0b10000000; }
  delay_us(97);

  // stop bit

  delay_us(100);

  rs232_send_char(c);
}



	
	
	
// THIS FUNCTION RECEIVES CHAR	--------------- MADE BY ABOHMEED

char rs232_recv_char() 
{
  unsigned char c = 0;

 

 if (RA0 == 1) 
 {delay_us(125);}


  if (RA0 == 0) { c = c | 0b00000001; }
  delay_us(97);

  if (RA0 == 0) { c = c | 0b00000010; }
  delay_us(97);

  if (RA0 == 0) { c = c | 0b00000100; }
  delay_us(97);

  if (RA0 == 0) { c = c | 0b00001000; }
  delay_us(97);

  if (RA0 == 0) { c = c | 0b00010000; }
  delay_us(97);

  if (RA0 == 0) { c = c | 0b00100000; }
  delay_us(97);

  if (RA0 == 0) { c = c | 0b01000000; }
  delay_us(97);

  if (RA0 == 0) { c = c | 0b10000000; }
  delay_us(97);

  // stop bit

  
  delay_us(100);

 
  return c ;
}
